﻿Type=Service
Version=3.5
@EndOfDesignText@
#Region  Service Attributes 
	#StartAtBoot: true
	#StartCommandReturnValue: android.app.Service.START_STICKY
#End Region

Sub Process_Globals
	Private VERSION As Double = 1.0
	Private wsh As WebSocketHandler
	Private id As String
	Private notif As Notification
	Public kvs As KeyValueStore
	Private pingTimer As Timer
	Private lastServerPong As Long
	Type Message (id As String, Text As String)
	Private ReconnectTimer As Timer
End Sub

Sub Service_Create
	
	'create a random id
	If File.Exists(File.DirInternal, "id.txt") = False Then
		File.WriteString(File.DirInternal, "id.txt", Rnd(0, 2147483647))
	End If
	id = File.ReadString(File.DirInternal, "id.txt")
	notif.Initialize
	notif.Sound = False
	notif.Icon = "icon"
	notif.Vibrate = False
	notif.AutoCancel = True
	kvs.Initialize(File.DirInternal, "msg.db")
	If kvs.ContainsKey("messages") = False Then
		Dim messages As List
		messages.Initialize
		kvs.PutObject("messages", messages)
	End If
	pingTimer.Initialize("pingTimer", 10000)
	ReconnectTimer.Initialize("ReconnectTimer", 5000)
End Sub

Sub Service_Start (StartingIntent As Intent)
	StartServiceAt(Me, DateTime.Now + 2 * DateTime.TicksPerMinute, False)
	If wsh.IsInitialized = False OR wsh.ws.Connected = False Then
		Connect
	End If
	If IsPaused(Main) = False Then
		CallSub2(Main, "UpdateId", id)
		CallSub2(Main, "UpdateStatus", wsh.ws.Connected)
		'Show stored messages
		Dim messages As List = kvs.GetObject("messages")
		If messages.Size > 0 Then
			CallSub2(Main, "NewMessage", messages)
			messages.Clear
			kvs.PutObject("messages", messages)
		End If
	End If
End Sub

Private Sub Connect
	Dim wsh As WebSocketHandler
	wsh.Initialize(Me, "wsh")
	wsh.Connect(Main.serverLink)
End Sub

Sub wsh_ServerReady(Params As List)
	Dim m As Map
	m.Initialize
	m.Put("id", id)
	m.Put("version", VERSION)
	wsh.SendEventToServer("Device_id", m)
End Sub

Sub wsh_NewMessage(Params As List)
	Dim messages As List
	messages.Initialize
	Dim ids As List
	ids.Initialize
	For i = 0 To Params.Size - 1 Step 2
		Dim m As Message
		m.Initialize
		m.id = Params.Get(i)
		m.Text = Params.Get(i + 1)
		messages.Add(m)
		ids.Add(m.id)
	Next
	
	If IsPaused(Main) = False Then
		CallSub2(Main, "NewMessage", messages)
	Else
		Dim msgs As List = kvs.GetObject("messages")
		msgs.AddAll(messages)
		kvs.PutObject("messages", msgs)
		notif.SetInfo("New Push Message", "There are " & msgs.Size & " new message(s)", Main)
		notif.Notify(1)
	End If
	Dim mm As Map
	mm.Initialize
	mm.Put("ids", ids)
	wsh.SendEventToServer("Device_MessageDelivered", mm)
End Sub

Sub wsh_Connected
	Log("WebSocket Connected")
	CallSub2(Main, "UpdateStatus", wsh.ws.Connected)
	pingTimer.Enabled = True
	lastServerPong = DateTime.Now
End Sub

Sub wsh_Closed (Reason As String)
	Log("WebSocket Closed: " & Reason)
	CallSub2(Main, "UpdateStatus", wsh.ws.Connected)
	pingTimer.Enabled = False
	'Try to connect again if the activity is visible.
	'Otherwise it will try to reconnect in a few minutes because of the StartServiceAt call
	If IsPaused(Main) = False Then ReconnectTimer.Enabled = True
End Sub

Sub ReconnectTimer_Tick
	If wsh.ws.Connected = False Then
		Connect
	End If
	ReconnectTimer.Enabled = False
End Sub

Sub PingTimer_Tick
	'Log("PingTimer_Tick: " & wsh.ws.Connected & ", " & lastServerPong & ", "  & (lastServerPong + 30 * DateTime.TicksPerSecond < DateTime.Now))
	If wsh.ws.Connected = False Then Return
	
	If lastServerPong + 30 * DateTime.TicksPerSecond < DateTime.Now Then
		'Log("Closing connection after 30 seconds of no response")
		wsh.Close
	Else
		Dim m As Map
		m.Initialize
		wsh.SendEventToServer("Device_Ping", m)
	End If
End Sub

Sub wsh_Pong(Params As List)
	lastServerPong = DateTime.Now
End Sub

Sub Service_Destroy
End Sub
